@@ -3,25 +3,23 @@
  *
  * begun 2005-09-15 rgerhards
  *
- * Copyright 2005-2008
- *     Rainer Gerhards and Adiscon GmbH. All Rights Reserved.
+ * Copyright 2005-2012 Adiscon GmbH.
  *
  * This file is part of rsyslog.
  *
- * Rsyslog is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * Rsyslog is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Rsyslog.  If not, see <http://www.gnu.org/licenses/>.
- *
- * A copy of the GPL can be found in the file "COPYING" in this distribution.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *       http://www.apache.org/licenses/LICENSE-2.0
+ *       -or-
+ *       see COPYING.ASL20 in the source distribution
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 #include "config.h"
 
@@ -210,22 +208,29 @@ rsRetVal parsSkipAfterChar(rsParsObj *pThis, char c)
 /* Skip whitespace. Often used to trim parsable entries.
  * Returns with ParsePointer set to first non-whitespace
  * character (or at end of string).
+ * If bRequireOne is set to true, at least one whitespace
+ * must exist, else an error is returned.
  */
 rsRetVal parsSkipWhitespace(rsParsObj *pThis)
 {
 	register unsigned char *pC;
+	int numSkipped;
+	DEFiRet;
+
 
 	rsCHECKVALIDOBJECT(pThis, OIDrsPars);
 
 	pC = rsCStrGetBufBeg(pThis->pCStr);
 
+	numSkipped = 0;
 	while(pThis->iCurrPos < rsCStrLen(pThis->pCStr)) {
 		if(!isspace((int)*(pC+pThis->iCurrPos)))
 			break;
 		++pThis->iCurrPos;
+		++numSkipped;
 	}
 
-	return RS_RET_OK;
+	RETiRet;
 }
 
 /* Parse string up to a delimiter.
