@@ -68,7 +68,6 @@ typedef struct _instanceData {
 	uchar *pwd;
 	uchar *dbNcoll;
 	uchar *tplName;
-	int bErrMsgPermitted;	/* only one errmsg permitted per connection */
 } instanceData;
 
 
@@ -140,21 +139,19 @@ static void
 reportMongoError(instanceData *pData)
 {
 	char errStr[1024];
+	errmsg.LogError(0, RS_RET_ERR, "ommongodb: error: %s",
+		rs_strerror_r(errno, errStr, sizeof(errStr)));
+#if 0
 	gchar *err;
-	int eno;
-
-	if(pData->bErrMsgPermitted) {
-		eno = errno;
-		if(mongo_sync_cmd_get_last_error(pData->conn, (gchar*)pData->db, &err) == TRUE) {
-			errmsg.LogError(0, RS_RET_ERR, "ommongodb: error: %s", err);
-		} else {
-			DBGPRINTF("ommongodb: we had an error, but can not obtain specifics, "
-				  "using plain old errno error message generator\n");
-			errmsg.LogError(0, RS_RET_ERR, "ommongodb: error: %s",
-				rs_strerror_r(eno, errStr, sizeof(errStr)));
-		}
-		pData->bErrMsgPermitted = 0;
+	if(mongo_sync_cmd_get_last_error(pData->conn, (gchar*)pData->db, &err) == TRUE) {
+		errmsg.LogError(0, RS_RET_ERR, "ommongodb: error: %s", err);
+	} else {
+		errmsg.LogError(0, RS_RET_ERR, "ommongodb: we had an error, but can "
+			"not obtain specifics");
 	}
+#else
+	(void)pData;
+#endif
 }
 
 
@@ -236,11 +233,11 @@ getDefaultBSON(msg_t *pMsg)
 	gint64 ts_gen, ts_rcv; /* timestamps: generated, received */
 	int secfrac;
 
-	procid = MsgGetProp(pMsg, NULL, PROP_PROGRAMNAME, NULL, &procid_len, &procid_free);
-	tag = MsgGetProp(pMsg, NULL, PROP_SYSLOGTAG, NULL, &tag_len, &tag_free);
-	pid = MsgGetProp(pMsg, NULL, PROP_PROCID, NULL, &pid_len, &pid_free);
-	sys = MsgGetProp(pMsg, NULL, PROP_HOSTNAME, NULL, &sys_len, &sys_free);
-	msg = MsgGetProp(pMsg, NULL, PROP_MSG, NULL, &msg_len, &msg_free);
+	procid = MsgGetProp(pMsg, NULL, PROP_PROGRAMNAME, NULL, &procid_len, &procid_free, NULL);
+	tag = MsgGetProp(pMsg, NULL, PROP_SYSLOGTAG, NULL, &tag_len, &tag_free, NULL);
+	pid = MsgGetProp(pMsg, NULL, PROP_PROCID, NULL, &pid_len, &pid_free, NULL);
+	sys = MsgGetProp(pMsg, NULL, PROP_HOSTNAME, NULL, &sys_len, &sys_free, NULL);
+	msg = MsgGetProp(pMsg, NULL, PROP_MSG, NULL, &msg_len, &msg_free, NULL);
 
 	// TODO: move to datetime? Refactor in any case! rgerhards, 2012-03-30
 	ts_gen = (gint64) datetime.syslogTime2time_t(&pMsg->tTIMESTAMP) * 1000; /* ms! */
@@ -436,11 +433,9 @@ CODESTARTdoAction
 		/* FIXME: is this a correct return code? */
 		ABORT_FINALIZE(RS_RET_ERR);
 	}
-	if(mongo_sync_cmd_insert(pData->conn, (char*)pData->dbNcoll, doc, NULL)) {
-		pData->bErrMsgPermitted = 1;
-	} else {
-		dbgprintf("ommongodb: insert error\n");
+	if(!mongo_sync_cmd_insert(pData->conn, (char*)pData->dbNcoll, doc, NULL)) {
 		reportMongoError(pData);
+		dbgprintf("ommongodb: insert error\n");
 		ABORT_FINALIZE(RS_RET_SUSPENDED);
 	}
 
