@@ -6,32 +6,28 @@
  * because the config file handler will by dynamically be loaded and be
  * kept in memory only as long as the config file is actually being 
  * processed. Thereafter, it shall be unloaded. -- rgerhards
+ * Please note that the original syslogd.c source was under BSD license
+ * at the time of the rsyslog fork from sysklogd.
  *
- * TODO: the license MUST be changed to LGPL. However, we can not
- * currently do that, because we use some sysklogd code to crunch
- * the selector lines (e.g. *.info). That code is scheduled for removal
- * as part of RainerScript. After this is done, we can change licenses.
- *
- * Copyright 2008-2011 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2008-2012 Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of rsyslog.
  *
- * Rsyslog is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * Rsyslog is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Rsyslog.  If not, see <http://www.gnu.org/licenses/>.
- *
- * A copy of the GPL can be found in the file "COPYING" in this distribution.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *       http://www.apache.org/licenses/LICENSE-2.0
+ *       -or-
+ *       see COPYING.ASL20 in the source distribution
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
-#define CFGLNSIZ 4096 /* the maximum size of a configuraton file line, after re-combination */
+#define CFGLNSIZ 64*1024 /* the maximum size of a configuraton file line, after re-combination */
 #include "config.h"
 #include <stdlib.h>
 #include <stdio.h>
@@ -65,17 +61,16 @@
 #include "srUtils.h"
 #include "errmsg.h"
 #include "net.h"
-#include "rule.h"
 #include "ruleset.h"
 #include "rsconf.h"
 #include "unicode-helper.h"
+#include "rainerscript.h"
 
 #ifdef OS_SOLARIS
 #	define NAME_MAX MAXNAMELEN
 #endif
 
 /* forward definitions */
-//static rsRetVal cfline(rsconf_t *conf, uchar *line, rule_t **pfCurr);
 
 
 /* static data */
@@ -83,10 +78,8 @@ DEFobjStaticHelpers
 DEFobjCurrIf(module)
 DEFobjCurrIf(errmsg)
 DEFobjCurrIf(net)
-DEFobjCurrIf(rule)
 DEFobjCurrIf(ruleset)
 
-ecslConfObjType currConfObj = eConfObjGlobal; /* to support scoping - which config object is currently active? */
 int bConfStrictScoping = 0;	/* force strict scoping during config processing? */
 
 
@@ -121,18 +114,15 @@ doModLoad(uchar **pp, __attribute__((unused)) void* pVal)
 	skipWhiteSpace(pp); /* skip over any whitespace */
 
 	/* this below is a quick and dirty hack to provide compatibility with the
-	 * $ModLoad MySQL forward compatibility statement. TODO: clean this up
-	 * For the time being, it is clean enough, it just needs to be done
-	 * differently when we have a full design for loadable plug-ins. For the
-	 * time being, we just mangle the names a bit.
-	 * rgerhards, 2007-08-14
+	 * $ModLoad MySQL forward compatibility statement. This needs to be supported
+	 * for legacy format.
 	 */
 	if(!strcmp((char*) szName, "MySQL"))
 		pModName = (uchar*) "ommysql.so";
 	else
 		pModName = szName;
 
-	CHKiRet(module.Load(pModName, 1));
+	CHKiRet(module.Load(pModName, 1, NULL));
 
 finalize_it:
 	RETiRet;
@@ -334,14 +324,9 @@ cflineParseFileName(uchar* p, uchar *pFileName, omodStringRequest_t *pOMSR, int
 }
 
 
-/* Helper to cfline(). This function takes the filter part of a traditional, PRI
- * based line and decodes the PRIs given in the selector line. It processed the
- * line up to the beginning of the action part. A pointer to that beginnig is
- * passed back to the caller.
- * rgerhards 2005-09-15
- */
+/* Decode a traditional PRI filter */
 /* GPLv3 - stems back to sysklogd */
-rsRetVal cflineProcessTradPRIFilter(uchar **pline, register rule_t *pRule)
+rsRetVal DecodePRIFilter(uchar *pline, uchar pmask[])
 {
 	uchar *p;
 	register uchar *q;
@@ -355,22 +340,15 @@ rsRetVal cflineProcessTradPRIFilter(uchar **pline, register rule_t *pRule)
 	DEFiRet;
 
 	ASSERT(pline != NULL);
-	ASSERT(*pline != NULL);
-	ISOBJ_TYPE_assert(pRule, rule);
 
-	dbgprintf(" - traditional PRI filter '%s'\n", *pline);
-	errno = 0;	/* keep strerror_r() stuff out of logerror messages */
+	dbgprintf("Decoding traditional PRI filter '%s'\n", pline);
 
-	pRule->f_filter_type = FILTER_PRI;
-	/* Note: file structure is pre-initialized to zero because it was
-	 * created with calloc()!
-	 */
 	for (i = 0; i <= LOG_NFACILITIES; i++) {
-		pRule->f_filterData.f_pmask[i] = TABLE_NOPRI;
+		pmask[i] = TABLE_NOPRI;
 	}
 
 	/* scan through the list of selectors */
-	for (p = *pline; *p && *p != '\t' && *p != ' ';) {
+	for (p = pline; *p && *p != '\t' && *p != ' ';) {
 		/* find the end of this facility name list */
 		for (q = p; *q && *q != '\t' && *q++ != '.'; )
 			continue;
@@ -419,28 +397,28 @@ rsRetVal cflineProcessTradPRIFilter(uchar **pline, register rule_t *pRule)
 				for (i = 0; i <= LOG_NFACILITIES; i++) {
 					if ( pri == INTERNAL_NOPRI ) {
 						if ( ignorepri )
-							pRule->f_filterData.f_pmask[i] = TABLE_ALLPRI;
+							pmask[i] = TABLE_ALLPRI;
 						else
-							pRule->f_filterData.f_pmask[i] = TABLE_NOPRI;
+							pmask[i] = TABLE_NOPRI;
 					}
 					else if ( singlpri ) {
 						if ( ignorepri )
-				  			pRule->f_filterData.f_pmask[i] &= ~(1<<pri);
+				  			pmask[i] &= ~(1<<pri);
 						else
-				  			pRule->f_filterData.f_pmask[i] |= (1<<pri);
+				  			pmask[i] |= (1<<pri);
 					} else {
 						if ( pri == TABLE_ALLPRI ) {
 							if ( ignorepri )
-								pRule->f_filterData.f_pmask[i] = TABLE_NOPRI;
+								pmask[i] = TABLE_NOPRI;
 							else
-								pRule->f_filterData.f_pmask[i] = TABLE_ALLPRI;
+								pmask[i] = TABLE_ALLPRI;
 						} else {
 							if ( ignorepri )
 								for (i2= 0; i2 <= pri; ++i2)
-									pRule->f_filterData.f_pmask[i] &= ~(1<<i2);
+									pmask[i] &= ~(1<<i2);
 							else
 								for (i2= 0; i2 <= pri; ++i2)
-									pRule->f_filterData.f_pmask[i] |= (1<<i2);
+									pmask[i] |= (1<<i2);
 						}
 					}
 				}
@@ -455,27 +433,27 @@ rsRetVal cflineProcessTradPRIFilter(uchar **pline, register rule_t *pRule)
 
 				if ( pri == INTERNAL_NOPRI ) {
 					if ( ignorepri )
-						pRule->f_filterData.f_pmask[i >> 3] = TABLE_ALLPRI;
+						pmask[i >> 3] = TABLE_ALLPRI;
 					else
-						pRule->f_filterData.f_pmask[i >> 3] = TABLE_NOPRI;
+						pmask[i >> 3] = TABLE_NOPRI;
 				} else if ( singlpri ) {
 					if ( ignorepri )
-						pRule->f_filterData.f_pmask[i >> 3] &= ~(1<<pri);
+						pmask[i >> 3] &= ~(1<<pri);
 					else
-						pRule->f_filterData.f_pmask[i >> 3] |= (1<<pri);
+						pmask[i >> 3] |= (1<<pri);
 				} else {
 					if ( pri == TABLE_ALLPRI ) {
 						if ( ignorepri )
-							pRule->f_filterData.f_pmask[i >> 3] = TABLE_NOPRI;
+							pmask[i >> 3] = TABLE_NOPRI;
 						else
-							pRule->f_filterData.f_pmask[i >> 3] = TABLE_ALLPRI;
+							pmask[i >> 3] = TABLE_ALLPRI;
 					} else {
 						if ( ignorepri )
 							for (i2= 0; i2 <= pri; ++i2)
-								pRule->f_filterData.f_pmask[i >> 3] &= ~(1<<i2);
+								pmask[i >> 3] &= ~(1<<i2);
 						else
 							for (i2= 0; i2 <= pri; ++i2)
-								pRule->f_filterData.f_pmask[i >> 3] |= (1<<i2);
+								pmask[i >> 3] |= (1<<i2);
 					}
 				}
 			}
@@ -486,11 +464,6 @@ rsRetVal cflineProcessTradPRIFilter(uchar **pline, register rule_t *pRule)
 		p = q;
 	}
 
-	/* skip to action part */
-	while (*p == '\t' || *p == ' ')
-		p++;
-
-	*pline = p;
 	RETiRet;
 }
 
@@ -500,7 +473,7 @@ rsRetVal cflineProcessTradPRIFilter(uchar **pline, register rule_t *pRule)
  * of the action part. A pointer to that beginnig is passed back to the caller.
  * rgerhards 2005-09-15
  */
-rsRetVal cflineProcessPropFilter(uchar **pline, register rule_t *f)
+rsRetVal DecodePropFilter(uchar *pline, struct cnfstmt *stmt)
 {
 	rsParsObj *pPars;
 	cstr_t *pCSCompOp;
@@ -509,16 +482,11 @@ rsRetVal cflineProcessPropFilter(uchar **pline, register rule_t *f)
 	int iOffset; /* for compare operations */
 
 	ASSERT(pline != NULL);
-	ASSERT(*pline != NULL);
-	ASSERT(f != NULL);
 
-	dbgprintf(" - property-based filter '%s'\n", *pline);
-	errno = 0;	/* keep strerror_r() stuff out of logerror messages */
-
-	f->f_filter_type = FILTER_PROP;
+	dbgprintf("Decoding property-based filter '%s'\n", pline);
 
 	/* create parser object starting with line string without leading colon */
-	if((iRet = rsParsConstructFromSz(&pPars, (*pline)+1)) != RS_RET_OK) {
+	if((iRet = rsParsConstructFromSz(&pPars, pline+1)) != RS_RET_OK) {
 		errmsg.LogError(0, iRet, "Error %d constructing parser object - ignoring selector", iRet);
 		return(iRet);
 	}
@@ -530,15 +498,15 @@ rsRetVal cflineProcessPropFilter(uchar **pline, register rule_t *f)
 		rsParsDestruct(pPars);
 		return(iRet);
 	}
-	iRet = propNameToID(pCSPropName, &f->f_filterData.prop.propID);
+	iRet = propNameToID(pCSPropName, &stmt->d.s_propfilt.propID);
 	if(iRet != RS_RET_OK) {
 		errmsg.LogError(0, iRet, "error %d parsing filter property - ignoring selector", iRet);
 		rsParsDestruct(pPars);
 		return(iRet);
 	}
-	if(f->f_filterData.prop.propID == PROP_CEE) {
+	if(stmt->d.s_propfilt.propID == PROP_CEE) {
 		/* in CEE case, we need to preserve the actual property name */
-		if((f->f_filterData.prop.propName =
+		if((stmt->d.s_propfilt.propName =
 		     es_newStrFromBuf((char*)cstrGetSzStrNoNULL(pCSPropName)+2, cstrLen(pCSPropName)-2)) == NULL) {
 			cstrDestruct(&pCSPropName);
 			return(RS_RET_ERR);
@@ -561,38 +529,38 @@ rsRetVal cflineProcessPropFilter(uchar **pline, register rule_t *f)
 	 */
 	if(rsCStrLen(pCSCompOp) > 0) {
 		if(*rsCStrGetBufBeg(pCSCompOp) == '!') {
-			f->f_filterData.prop.isNegated = 1;
+			stmt->d.s_propfilt.isNegated = 1;
 			iOffset = 1; /* ignore '!' */
 		} else {
-			f->f_filterData.prop.isNegated = 0;
+			stmt->d.s_propfilt.isNegated = 0;
 			iOffset = 0;
 		}
 	} else {
-		f->f_filterData.prop.isNegated = 0;
+		stmt->d.s_propfilt.isNegated = 0;
 		iOffset = 0;
 	}
 
 	if(!rsCStrOffsetSzStrCmp(pCSCompOp, iOffset, (uchar*) "contains", 8)) {
-		f->f_filterData.prop.operation = FIOP_CONTAINS;
+		stmt->d.s_propfilt.operation = FIOP_CONTAINS;
 	} else if(!rsCStrOffsetSzStrCmp(pCSCompOp, iOffset, (uchar*) "isequal", 7)) {
-		f->f_filterData.prop.operation = FIOP_ISEQUAL;
+		stmt->d.s_propfilt.operation = FIOP_ISEQUAL;
 	} else if(!rsCStrOffsetSzStrCmp(pCSCompOp, iOffset, (uchar*) "isempty", 7)) {
-		f->f_filterData.prop.operation = FIOP_ISEMPTY;
+		stmt->d.s_propfilt.operation = FIOP_ISEMPTY;
 	} else if(!rsCStrOffsetSzStrCmp(pCSCompOp, iOffset, (uchar*) "startswith", 10)) {
-		f->f_filterData.prop.operation = FIOP_STARTSWITH;
+		stmt->d.s_propfilt.operation = FIOP_STARTSWITH;
 	} else if(!rsCStrOffsetSzStrCmp(pCSCompOp, iOffset, (unsigned char*) "regex", 5)) {
-		f->f_filterData.prop.operation = FIOP_REGEX;
+		stmt->d.s_propfilt.operation = FIOP_REGEX;
 	} else if(!rsCStrOffsetSzStrCmp(pCSCompOp, iOffset, (unsigned char*) "ereregex", 8)) {
-		f->f_filterData.prop.operation = FIOP_EREREGEX;
+		stmt->d.s_propfilt.operation = FIOP_EREREGEX;
 	} else {
 		errmsg.LogError(0, NO_ERRCODE, "error: invalid compare operation '%s' - ignoring selector",
 		           (char*) rsCStrGetSzStrNoNULL(pCSCompOp));
 	}
 	rsCStrDestruct(&pCSCompOp); /* no longer needed */
 
-	if(f->f_filterData.prop.operation != FIOP_ISEMPTY) {
+	if(stmt->d.s_propfilt.operation != FIOP_ISEMPTY) {
 		/* read compare value */
-		iRet = parsQuotedCStr(pPars, &f->f_filterData.prop.pCSCompValue);
+		iRet = parsQuotedCStr(pPars, &stmt->d.s_propfilt.pCSCompValue);
 		if(iRet != RS_RET_OK) {
 			errmsg.LogError(0, iRet, "error %d compare value property - ignoring selector", iRet);
 			rsParsDestruct(pPars);
@@ -600,114 +568,10 @@ rsRetVal cflineProcessPropFilter(uchar **pline, register rule_t *f)
 		}
 	}
 
-	/* skip to action part */
-	if((iRet = parsSkipWhitespace(pPars)) != RS_RET_OK) {
-		errmsg.LogError(0, iRet, "error %d skipping to action part - ignoring selector", iRet);
-		rsParsDestruct(pPars);
-		return(iRet);
-	}
-
-	/* cleanup */
-	*pline = *pline + rsParsGetParsePointer(pPars) + 1;
-		/* we are adding one for the skipped initial ":" */
-
 	return rsParsDestruct(pPars);
 }
 
 
-/*
- * Helper to cfline(). This function interprets a BSD host selector line
- * from the config file ("+/-hostname"). It stores it for further reference.
- * rgerhards 2005-10-19
- */
-rsRetVal cflineProcessHostSelector(uchar **pline)
-{
-	DEFiRet;
-
-	ASSERT(pline != NULL);
-	ASSERT(*pline != NULL);
-	ASSERT(**pline == '-' || **pline == '+');
-
-	dbgprintf(" - host selector line\n");
-
-	/* check include/exclude setting */
-	if(**pline == '+') {
-		eDfltHostnameCmpMode = HN_COMP_MATCH;
-	} else { /* we do not check for '-', it must be, else we wouldn't be here */
-		eDfltHostnameCmpMode = HN_COMP_NOMATCH;
-	}
-	(*pline)++;	/* eat + or - */
-
-	/* the below is somewhat of a quick hack, but it is efficient (this is
-	 * why it is in here. "+*" resets the tag selector with BSD syslog. We mimic
-	 * this, too. As it is easy to check that condition, we do not fire up a
-	 * parser process, just make sure we do not address beyond our space.
-	 * Order of conditions in the if-statement is vital! rgerhards 2005-10-18
-	 */
-	if(**pline != '\0' && **pline == '*' && *(*pline+1) == '\0') {
-		dbgprintf("resetting BSD-like hostname filter\n");
-		eDfltHostnameCmpMode = HN_NO_COMP;
-		if(pDfltHostnameCmp != NULL) {
-			CHKiRet(rsCStrSetSzStr(pDfltHostnameCmp, NULL));
-		}
-	} else {
-		dbgprintf("setting BSD-like hostname filter to '%s'\n", *pline);
-		if(pDfltHostnameCmp == NULL) {
-			/* create string for parser */
-			CHKiRet(rsCStrConstructFromszStr(&pDfltHostnameCmp, *pline));
-		} else { /* string objects exists, just update... */
-			CHKiRet(rsCStrSetSzStr(pDfltHostnameCmp, *pline));
-		}
-	}
-
-finalize_it:
-	RETiRet;
-}
-
-
-/*
- * Helper to cfline(). This function interprets a BSD tag selector line
- * from the config file ("!tagname"). It stores it for further reference.
- * rgerhards 2005-10-18
- */
-rsRetVal cflineProcessTagSelector(uchar **pline)
-{
-	DEFiRet;
-
-	ASSERT(pline != NULL);
-	ASSERT(*pline != NULL);
-	ASSERT(**pline == '!');
-
-	dbgprintf(" - programname selector line\n");
-
-	(*pline)++;	/* eat '!' */
-
-	/* the below is somewhat of a quick hack, but it is efficient (this is
-	 * why it is in here. "!*" resets the tag selector with BSD syslog. We mimic
-	 * this, too. As it is easy to check that condition, we do not fire up a
-	 * parser process, just make sure we do not address beyond our space.
-	 * Order of conditions in the if-statement is vital! rgerhards 2005-10-18
-	 */
-	if(**pline != '\0' && **pline == '*' && *(*pline+1) == '\0') {
-		dbgprintf("resetting programname filter\n");
-		if(pDfltProgNameCmp != NULL) {
-			rsCStrDestruct(&pDfltProgNameCmp);
-		}
-	} else {
-		dbgprintf("setting programname filter to '%s'\n", *pline);
-		if(pDfltProgNameCmp == NULL) {
-			/* create string for parser */
-			CHKiRet(rsCStrConstructFromszStr(&pDfltProgNameCmp, *pline));
-		} else { /* string objects exists, just update... */
-			CHKiRet(rsCStrSetSzStr(pDfltProgNameCmp, *pline));
-		}
-	}
-
-finalize_it:
-	RETiRet;
-}
-
-
 /* process the action part of a selector line
  * rgerhards, 2007-08-01
  */
@@ -716,6 +580,7 @@ rsRetVal cflineDoAction(rsconf_t *conf, uchar **p, action_t **ppAction)
 	modInfo_t *pMod;
 	cfgmodules_etry_t *node;
 	omodStringRequest_t *pOMSR;
+	int bHadWarning = 0;
 	action_t *pAction = NULL;
 	void *pModData;
 	DEFiRet;
@@ -735,12 +600,11 @@ rsRetVal cflineDoAction(rsconf_t *conf, uchar **p, action_t **ppAction)
 		pMod = node->pMod;
 		iRet = pMod->mod.om.parseSelectorAct(p, &pModData, &pOMSR);
 		dbgprintf("tried selector action for %s: %d\n", module.GetName(pMod), iRet);
+		if(iRet == RS_RET_OK_WARN) {
+			bHadWarning = 1;
+			iRet = RS_RET_OK;
+		}
 		if(iRet == RS_RET_OK || iRet == RS_RET_SUSPENDED) {
-			/* advance our config parser state: we now only accept an $End as valid,
-			 * no more action statments.
-			 */
-			if(currConfObj == eConfObjAction)
-				currConfObj = eConfObjActionWaitEnd;
 			if((iRet = addAction(&pAction, pMod, pModData, pOMSR, NULL, NULL,
 					     (iRet == RS_RET_SUSPENDED)? 1 : 0)) == RS_RET_OK) {
 				/* now check if the module is compatible with select features */
@@ -754,8 +618,7 @@ rsRetVal cflineDoAction(rsconf_t *conf, uchar **p, action_t **ppAction)
 				conf->actions.nbrActions++;	/* one more active action! */
 			}
 			break;
-		}
-		else if(iRet != RS_RET_CONFLINE_UNPROCESSED) {
+		} else if(iRet != RS_RET_CONFLINE_UNPROCESSED) {
 			/* In this case, the module would have handled the config
 			 * line, but some error occured while doing so. This error should
 			 * already by reported by the module. We do not try any other
@@ -769,6 +632,8 @@ rsRetVal cflineDoAction(rsconf_t *conf, uchar **p, action_t **ppAction)
 	}
 
 	*ppAction = pAction;
+	if(iRet == RS_RET_OK && bHadWarning)
+		iRet = RS_RET_OK_WARN;
 	RETiRet;
 }
 
@@ -809,131 +674,12 @@ finalize_it:
 ENDobjQueryInterface(conf)
 
 
-/* switch to a new action scope. This means that we switch the current 
- * mode to action, but it also means we need to clear all scope variables,
- * so that we have a new environment.
- * rgerhards, 2010-07-23
- */
-static inline rsRetVal
-setActionScope(void)
-{
-	DEFiRet;
-	cfgmodules_etry_t *node;
-
-	currConfObj = eConfObjAction;
-	DBGPRINTF("entering action scope\n");
-	CHKiRet(actionNewScope());
-
-	/* now tell each action to start the scope */
-	node = NULL;
-	while((node = module.GetNxtCnfType(loadConf, node, eMOD_OUT)) != NULL) {
-		DBGPRINTF("beginning scope on module %s\n", node->pMod->pszName);
-		node->pMod->mod.om.newScope();
-	}
-
-finalize_it:
-	RETiRet;
-}
-
-
-/* switch back from action scope.
- * rgerhards, 2010-07-27
- */
-static inline rsRetVal
-unsetActionScope(void)
-{
-	DEFiRet;
-	cfgmodules_etry_t *node;
-
-	currConfObj = eConfObjAction;
-	DBGPRINTF("exiting action scope\n");
-	CHKiRet(actionRestoreScope());
-
-	/* now tell each action to restore the scope */
-	node = NULL;
-	while((node = module.GetNxtCnfType(loadConf, node, eMOD_OUT)) != NULL) {
-		DBGPRINTF("exiting scope on module %s\n", node->pMod->pszName);
-		node->pMod->mod.om.restoreScope();
-	}
-
-finalize_it:
-	RETiRet;
-}
-
-
-/* This method is called by our own handlers to begin a new config
- * object ($Begin statement). This also implies a new scope.
- * rgerhards, 2010-07-23
- */
-static rsRetVal
-beginConfObj(void __attribute__((unused)) *pVal, uchar *pszName)
-{
-	DEFiRet;
-
-	if(currConfObj != eConfObjGlobal) {
-		errmsg.LogError(0, RS_RET_CONF_NOT_GLBL, "not in global scope - can not nest $Begin");
-		ABORT_FINALIZE(RS_RET_CONF_NOT_GLBL);
-	}
-
-	if(!strcasecmp((char*)pszName, "action")) {
-		setActionScope();
-	} else {
-		errmsg.LogError(0, RS_RET_INVLD_CONF_OBJ, "invalid config object \"%s\" in $Begin", pszName);
-		ABORT_FINALIZE(RS_RET_INVLD_CONF_OBJ);
-	}
-
-finalize_it:
-	free(pszName); /* no longer needed */
-	RETiRet;
-}
-
-
-/* This method is called to end a config scope and switch
- * back to global scope.
- * rgerhards, 2010-07-23
- */
-static rsRetVal
-endConfObj(void __attribute__((unused)) *pVal, uchar *pszName)
-{
-	DEFiRet;
-
-	if(currConfObj == eConfObjGlobal) {
-		errmsg.LogError(0, RS_RET_CONF_NOT_GLBL, "already in global scope - dangling $End");
-		ABORT_FINALIZE(RS_RET_CONF_IN_GLBL);
-	}
-
-	if(!strcasecmp((char*)pszName, "action")) {
-		if(currConfObj == eConfObjAction) {
-			errmsg.LogError(0, RS_RET_CONF_END_NO_ACT, "$End action but not action specified");
-			/* this is a warning, we continue processing in that case (unscope) */
-		} else if(currConfObj != eConfObjActionWaitEnd) {
-			errmsg.LogError(0, RS_RET_CONF_INVLD_END, "$End not for active config object - "
-							          "nesting error?");
-			ABORT_FINALIZE(RS_RET_CONF_INVLD_END);
-		}
-		currConfObj = eConfObjGlobal;
-		CHKiRet(unsetActionScope());
-	} else {
-		errmsg.LogError(0, RS_RET_INVLD_CONF_OBJ, "invalid config object \"%s\" in $End", pszName);
-		ABORT_FINALIZE(RS_RET_INVLD_CONF_OBJ);
-	}
-
-finalize_it:
-	free(pszName); /* no longer needed */
-	RETiRet;
-}
-
-
-/* Reset config variables to default values. Note that
- * when we are inside an scope, we simply reset this to global.
- * However, $ResetConfigVariables is a global directive, and as such
- * will not be honored inside a scope!
+/* Reset config variables to default values.
  * rgerhards, 2010-07-23
  */
 static rsRetVal
 resetConfigVariables(uchar __attribute__((unused)) *pp, void __attribute__((unused)) *pVal)
 {
-	currConfObj = eConfObjGlobal;
 	bConfStrictScoping = 0;
 	return RS_RET_OK;
 }
@@ -957,7 +703,6 @@ CODESTARTObjClassExit(conf)
 	objRelease(module, CORE_COMPONENT);
 	objRelease(errmsg, CORE_COMPONENT);
 	objRelease(net, LM_NET_FILENAME);
-	objRelease(rule, CORE_COMPONENT);
 	objRelease(ruleset, CORE_COMPONENT);
 ENDObjClassExit(conf)
 
@@ -971,13 +716,12 @@ BEGINAbstractObjClassInit(conf, 1, OBJ_IS_CORE_MODULE) /* class, version - CHANG
 	CHKiRet(objUse(module, CORE_COMPONENT));
 	CHKiRet(objUse(errmsg, CORE_COMPONENT));
 	CHKiRet(objUse(net, LM_NET_FILENAME)); /* TODO: make this dependcy go away! */
-	CHKiRet(objUse(rule, CORE_COMPONENT));
 	CHKiRet(objUse(ruleset, CORE_COMPONENT));
 
-	CHKiRet(regCfSysLineHdlr((uchar *)"begin", 0, eCmdHdlrGetWord, beginConfObj, NULL, NULL, eConfObjGlobal));
-	CHKiRet(regCfSysLineHdlr((uchar *)"end", 0, eCmdHdlrGetWord, endConfObj, NULL, NULL, eConfObjAlways));
-	CHKiRet(regCfSysLineHdlr((uchar *)"strictscoping", 0, eCmdHdlrBinary, NULL, &bConfStrictScoping, NULL, eConfObjGlobal));
-	CHKiRet(regCfSysLineHdlr((uchar *)"resetconfigvariables", 1, eCmdHdlrCustomHandler, resetConfigVariables, NULL, NULL, eConfObjAction));
+ 	/* These commands will NOT be supported -- the new v6.3 config system provides
+	 * far better methods. We will remove the related code soon. -- rgerhards, 2012-01-09
+	 */
+	CHKiRet(regCfSysLineHdlr((uchar *)"resetconfigvariables", 1, eCmdHdlrCustomHandler, resetConfigVariables, NULL, NULL));
 ENDObjClassInit(conf)
 
 /* vi:set ai:
