@@ -143,6 +143,14 @@ finalize_it:
 	RETiRet;
 }
 
+void
+printParserList(parserList_t *pList)
+{
+	while(pList != NULL) {
+		dbgprintf("parser: %s\n", pList->pParser->pName);
+		pList = pList->pNext;
+	}
+}
 
 /* find a parser based on the provided name */
 static rsRetVal
@@ -180,7 +188,7 @@ AddDfltParser(uchar *pName)
 
 	CHKiRet(FindParser(&pParser, pName));
 	CHKiRet(AddParserToList(&pDfltParsLst, pParser));
-	dbgprintf("Parser '%s' added to default parser set.\n", pName);
+	DBGPRINTF("Parser '%s' added to default parser set.\n", pName);
 	
 finalize_it:
 	RETiRet;
@@ -209,7 +217,7 @@ finalize_it:
 
 BEGINobjDestruct(parser) /* be sure to specify the object type also in END and CODESTART macros! */
 CODESTARTobjDestruct(parser)
-	dbgprintf("destructing parser '%s'\n", pThis->pName);
+	DBGPRINTF("destructing parser '%s'\n", pThis->pName);
 	free(pThis->pName);
 ENDobjDestruct(parser)
 
@@ -311,7 +319,7 @@ SanitizeMsg(msg_t *pMsg)
 	size_t iDst;
 	size_t iMaxLine;
 	size_t maxDest;
-	sbool bUpdatedLen = FALSE;
+	sbool bUpdatedLen = RSFALSE;
 	uchar szSanBuf[32*1024]; /* buffer used for sanitizing a string */
 
 	assert(pMsg != NULL);
@@ -326,7 +334,7 @@ SanitizeMsg(msg_t *pMsg)
 	 */
 	if(pszMsg[lenMsg-1] == '\0') {
 		DBGPRINTF("dropped NUL at very end of message\n");
-		bUpdatedLen = TRUE;
+		bUpdatedLen = RSTRUE;
 		lenMsg--;
 	}
 
@@ -339,7 +347,7 @@ SanitizeMsg(msg_t *pMsg)
 		DBGPRINTF("dropped LF at very end of message (DropTrailingLF is set)\n");
 		lenMsg--;
 		pszMsg[lenMsg] = '\0';
-		bUpdatedLen = TRUE;
+		bUpdatedLen = RSTRUE;
 	}
 
 	/* it is much quicker to sweep over the message and see if it actually
@@ -370,7 +378,7 @@ SanitizeMsg(msg_t *pMsg)
 	}
 
 	if(!bNeedSanitize) {
-		if(bUpdatedLen == TRUE)
+		if(bUpdatedLen == RSTRUE)
 			MsgSetRawMsgSize(pMsg, lenMsg);
 		FINALIZE;
 	}
@@ -508,20 +516,20 @@ ParseMsg(msg_t *pMsg)
 	DBGPRINTF("parse using parser list %p%s.\n", pParserList,
 		  (pParserList == pDfltParsLst) ? " (the default list)" : "");
 
-	bIsSanitized = FALSE;
-	bPRIisParsed = FALSE;
+	bIsSanitized = RSFALSE;
+	bPRIisParsed = RSFALSE;
 	while(pParserList != NULL) {
 		pParser = pParserList->pParser;
-		if(pParser->bDoSanitazion && bIsSanitized == FALSE) {
+		if(pParser->bDoSanitazion && bIsSanitized == RSFALSE) {
 			CHKiRet(SanitizeMsg(pMsg));
-			if(pParser->bDoPRIParsing && bPRIisParsed == FALSE) {
+			if(pParser->bDoPRIParsing && bPRIisParsed == RSFALSE) {
 				CHKiRet(ParsePRI(pMsg));
-				bPRIisParsed = TRUE;
+				bPRIisParsed = RSTRUE;
 			}
-			bIsSanitized = TRUE;
+			bIsSanitized = RSTRUE;
 		}
 		localRet = pParser->pModule->mod.pm.parse(pMsg);
-		dbgprintf("Parser '%s' returned %d\n", pParser->pName, localRet);
+		DBGPRINTF("Parser '%s' returned %d\n", pParser->pName, localRet);
 		if(localRet != RS_RET_COULD_NOT_PARSE)
 			break;
 		pParserList = pParserList->pNext;
