@@ -67,7 +67,6 @@
 #include "dirty.h"
 #include "template.h"
 
-extern char* yytext;
 /* static data */
 DEFobjStaticHelpers
 DEFobjCurrIf(ruleset)
@@ -391,7 +390,7 @@ parser_errmsg(char *fmt, ...)
 int
 yyerror(char *s)
 {
-	parser_errmsg("%s on token '%s'", s, yytext);
+	parser_errmsg("%s", s);
 	return 0;
 }
 void cnfDoObj(struct cnfobj *o)
