@@ -378,7 +378,8 @@ evalPROPFILT(struct cnfstmt *stmt, msg_t *pMsg)
 		goto done;
 
 	pszPropVal = MsgGetProp(pMsg, NULL, stmt->d.s_propfilt.propID,
-				stmt->d.s_propfilt.propName, &propLen, &pbMustBeFreed);
+				stmt->d.s_propfilt.propName, &propLen,
+				&pbMustBeFreed, NULL);
 
 	/* Now do the compares (short list currently ;)) */
 	switch(stmt->d.s_propfilt.operation ) {
@@ -458,12 +459,14 @@ static void
 execPROPFILT(struct cnfstmt *stmt, batch_t *pBatch, sbool *active)
 {
 	sbool *thenAct;
+	msg_t *pMsg;
 	sbool bRet;
 	int i;
 	thenAct = newActive(pBatch);
 	for(i = 0 ; i < batchNumMsgs(pBatch) && !*(pBatch->pbShutdownImmediate) ; ++i) {
 		if(pBatch->pElem[i].state == BATCH_STATE_DISC)
 			continue; /* will be ignored in any case */
+		pMsg = (msg_t*)(pBatch->pElem[i].pUsrp);
 		if(active == NULL || active[i]) {
 			bRet = evalPROPFILT(stmt, (msg_t*)(pBatch->pElem[i].pUsrp));
 		} else 
