@@ -141,7 +141,9 @@ finalize_it:
  * offers big performance improvements.
  * rewritten 2009-06-19 rgerhards
  */
-rsRetVal tplToString(struct template *pTpl, msg_t *pMsg, uchar **ppBuf, size_t *pLenBuf)
+rsRetVal
+tplToString(struct template *pTpl, msg_t *pMsg, uchar **ppBuf, size_t *pLenBuf,
+	    struct syslogTime *ttNow)
 {
 	DEFiRet;
 	struct templateEntry *pTpe;
@@ -191,7 +193,8 @@ rsRetVal tplToString(struct template *pTpl, msg_t *pMsg, uchar **ppBuf, size_t *
 			bMustBeFreed = 0;
 		} else 	if(pTpe->eEntryType == FIELD) {
 			pVal = (uchar*) MsgGetProp(pMsg, pTpe, pTpe->data.field.propid,
-						   pTpe->data.field.propName,  &iLenVal, &bMustBeFreed);
+						   pTpe->data.field.propName,  &iLenVal,
+						   &bMustBeFreed, ttNow);
 			/* we now need to check if we should use SQL option. In this case,
 			 * we must go over the generated string and escape '\'' characters.
 			 * rgerhards, 2005-09-22: the option values below look somewhat misplaced,
@@ -245,7 +248,8 @@ finalize_it:
  * is indicated by a NULL pointer.
  * rgerhards, 2009-04-03
  */
-rsRetVal tplToArray(struct template *pTpl, msg_t *pMsg, uchar*** ppArr)
+rsRetVal
+tplToArray(struct template *pTpl, msg_t *pMsg, uchar*** ppArr, struct syslogTime *ttNow)
 {
 	DEFiRet;
 	struct templateEntry *pTpe;
@@ -286,7 +290,8 @@ rsRetVal tplToArray(struct template *pTpl, msg_t *pMsg, uchar*** ppArr)
 			CHKmalloc(pArr[iArr] = (uchar*)strdup((char*) pTpe->data.constant.pConstant));
 		} else 	if(pTpe->eEntryType == FIELD) {
 			pVal = (uchar*) MsgGetProp(pMsg, pTpe, pTpe->data.field.propid,
-						   pTpe->data.field.propName,  &propLen, &bMustBeFreed);
+						   pTpe->data.field.propName,  &propLen,
+						   &bMustBeFreed, ttNow);
 			if(bMustBeFreed) { /* if it must be freed, it is our own private copy... */
 				pArr[iArr] = pVal; /* ... so we can use it! */
 			} else {
@@ -310,7 +315,7 @@ finalize_it:
  * rgerhards, 2012-08-29
  */
 rsRetVal
-tplToJSON(struct template *pTpl, msg_t *pMsg, struct json_object **pjson)
+tplToJSON(struct template *pTpl, msg_t *pMsg, struct json_object **pjson, struct syslogTime *ttNow)
 {
 	struct templateEntry *pTpe;
 	rs_size_t propLen;
@@ -353,7 +358,7 @@ tplToJSON(struct template *pTpl, msg_t *pMsg, struct json_object **pjson)
 			} else  {
 				pVal = (uchar*) MsgGetProp(pMsg, pTpe, pTpe->data.field.propid,
 							   pTpe->data.field.propName,  &propLen,
-							   &bMustBeFreed);
+							   &bMustBeFreed, ttNow);
 				if(pTpe->data.field.options.bMandatory || propLen > 0) {
 					jsonf = json_object_new_string_len((char*)pVal, propLen);
 					json_object_object_add(json, (char*)pTpe->fieldName, jsonf);
@@ -371,6 +376,38 @@ finalize_it:
 }
 
 
+/* Check if the template requires a date call (actually a cached
+ * date structure). This currently is the case for the $NOW family
+ * of properties.
+ */
+int
+tplRequiresDateCall(struct template *pTpl)
+{
+	struct templateEntry *pTpe;
+	int r = 0;
+
+	if(pTpl->subtree != NULL)
+		goto done;
+
+	for(pTpe = pTpl->pEntryRoot ; pTpe != NULL ; pTpe = pTpe->pNext) {
+		switch(pTpe->data.field.propid) {
+		case PROP_SYS_NOW:
+		case PROP_SYS_YEAR:
+		case PROP_SYS_MONTH:
+		case PROP_SYS_DAY:
+		case PROP_SYS_HOUR:
+		case PROP_SYS_HHOUR:
+		case PROP_SYS_QHOUR:
+		case PROP_SYS_MINUTE:
+			r = 1;
+			goto done;
+		default:break;
+		}
+	}
+done:	return r;
+}
+
+
 /* Helper to doEscape. This is called if doEscape
  * runs out of memory allocating the escaped string.
  * Then we are in trouble. We can
@@ -791,6 +828,7 @@ static int do_Parameter(unsigned char **pp, struct template *pTpl)
 
 	/* Check frompos, if it has an R, then topos should be a regex */
 	if(*p == ':') {
+		pTpe->bComplexProcessing = 1;
 		++p; /* eat ':' */
 #ifdef FEATURE_REGEXP
 		if(*p == 'R') {
@@ -1351,6 +1389,7 @@ createPropertyTpe(struct template *pTpl, struct cnfobj *o)
 	int fielddelim = 9; /* default is HT (USACSII 9) */
 	int re_matchToUse = 0;
 	int re_submatchToUse = 0;
+	int bComplexProcessing = 0;
 	char *re_expr = NULL;
 	struct cnfparamvals *pvals = NULL;
 	enum {F_NONE, F_CSV, F_JSON, F_JSONF} formatType = F_NONE;
@@ -1376,23 +1415,31 @@ createPropertyTpe(struct template *pTpl, struct cnfobj *o)
 			free(tmpstr);
 		} else if(!strcmp(pblkProperty.descr[i].name, "droplastlf")) {
 			droplastlf = pvals[i].val.d.n;
+			bComplexProcessing = 1;
 		} else if(!strcmp(pblkProperty.descr[i].name, "mandatory")) {
 			mandatory = pvals[i].val.d.n;
 		} else if(!strcmp(pblkProperty.descr[i].name, "spifno1stsp")) {
 			spifno1stsp = pvals[i].val.d.n;
+			bComplexProcessing = 1;
 		} else if(!strcmp(pblkProperty.descr[i].name, "outname")) {
 			outname = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
 		} else if(!strcmp(pblkProperty.descr[i].name, "position.from")) {
 			frompos = pvals[i].val.d.n;
+			bComplexProcessing = 1;
 		} else if(!strcmp(pblkProperty.descr[i].name, "position.to")) {
 			topos = pvals[i].val.d.n;
+			bComplexProcessing = 1;
 		} else if(!strcmp(pblkProperty.descr[i].name, "field.number")) {
 			fieldnum = pvals[i].val.d.n;
+			bComplexProcessing = 1;
 		} else if(!strcmp(pblkProperty.descr[i].name, "field.delimiter")) {
 			fielddelim = pvals[i].val.d.n;
+			bComplexProcessing = 1;
 		} else if(!strcmp(pblkProperty.descr[i].name, "regex.expression")) {
 			re_expr = es_str2cstr(pvals[i].val.d.estr, NULL);
+			bComplexProcessing = 1;
 		} else if(!strcmp(pblkProperty.descr[i].name, "regex.type")) {
+			bComplexProcessing = 1;
 			if(!es_strbufcmp(pvals[i].val.d.estr, (uchar*)"BRE", sizeof("BRE")-1)) {
 				re_type = TPL_REGEX_BRE;
 			} else if(!es_strbufcmp(pvals[i].val.d.estr, (uchar*)"ERE", sizeof("ERE")-1)) {
@@ -1405,6 +1452,7 @@ createPropertyTpe(struct template *pTpl, struct cnfobj *o)
 				ABORT_FINALIZE(RS_RET_ERR);
 			}
 		} else if(!strcmp(pblkProperty.descr[i].name, "regex.nomatchmode")) {
+			bComplexProcessing = 1;
 			if(!es_strbufcmp(pvals[i].val.d.estr, (uchar*)"DFLT", sizeof("DFLT")-1)) {
 				re_nomatchType = TPL_REGEX_NOMATCH_USE_DFLTSTR;
 			} else if(!es_strbufcmp(pvals[i].val.d.estr, (uchar*)"BLANK", sizeof("BLANK")-1)) {
@@ -1421,10 +1469,13 @@ createPropertyTpe(struct template *pTpl, struct cnfobj *o)
 				ABORT_FINALIZE(RS_RET_ERR);
 			}
 		} else if(!strcmp(pblkProperty.descr[i].name, "regex.match")) {
+			bComplexProcessing = 1;
 			re_matchToUse = pvals[i].val.d.n;
 		} else if(!strcmp(pblkProperty.descr[i].name, "regex.submatch")) {
+			bComplexProcessing = 1;
 			re_submatchToUse = pvals[i].val.d.n;
 		} else if(!strcmp(pblkProperty.descr[i].name, "format")) {
+			bComplexProcessing = 1;
 			if(!es_strbufcmp(pvals[i].val.d.estr, (uchar*)"csv", sizeof("csv")-1)) {
 				formatType = F_CSV;
 			} else if(!es_strbufcmp(pvals[i].val.d.estr, (uchar*)"json", sizeof("json")-1)) {
@@ -1439,6 +1490,7 @@ createPropertyTpe(struct template *pTpl, struct cnfobj *o)
 				ABORT_FINALIZE(RS_RET_ERR);
 			}
 		} else if(!strcmp(pblkProperty.descr[i].name, "controlcharacters")) {
+			bComplexProcessing = 1;
 			if(!es_strbufcmp(pvals[i].val.d.estr, (uchar*)"escape", sizeof("escape")-1)) {
 				controlchr = CC_ESCAPE;
 			} else if(!es_strbufcmp(pvals[i].val.d.estr, (uchar*)"space", sizeof("space")-1)) {
@@ -1453,6 +1505,7 @@ createPropertyTpe(struct template *pTpl, struct cnfobj *o)
 				ABORT_FINALIZE(RS_RET_ERR);
 			}
 		} else if(!strcmp(pblkProperty.descr[i].name, "securepath")) {
+			bComplexProcessing = 1;
 			if(!es_strbufcmp(pvals[i].val.d.estr, (uchar*)"drop", sizeof("drop")-1)) {
 				secpath = SP_DROP;
 			} else if(!es_strbufcmp(pvals[i].val.d.estr, (uchar*)"replace", sizeof("replace")-1)) {
@@ -1465,6 +1518,7 @@ createPropertyTpe(struct template *pTpl, struct cnfobj *o)
 				ABORT_FINALIZE(RS_RET_ERR);
 			}
 		} else if(!strcmp(pblkProperty.descr[i].name, "caseconversion")) {
+			bComplexProcessing = 1;
 			if(!es_strbufcmp(pvals[i].val.d.estr, (uchar*)"lower", sizeof("lower")-1)) {
 				caseconv = tplCaseConvLower;
 			} else if(!es_strbufcmp(pvals[i].val.d.estr, (uchar*)"upper", sizeof("upper")-1)) {
@@ -1583,6 +1637,7 @@ createPropertyTpe(struct template *pTpl, struct cnfobj *o)
 	pTpe->fieldName = outname;
 	if(outname != NULL)
 		pTpe->lenFieldName = ustrlen(outname);
+	pTpe->bComplexProcessing = bComplexProcessing;
 	pTpe->data.field.eDateFormat = datefmt;
 	if(fieldnum != -1) {
 		pTpe->data.field.has_fields = 1;
@@ -2117,6 +2172,8 @@ void tplPrintList(rsconf_t *conf)
 				}
 				break;
 			}
+			if(pTpe->bComplexProcessing)
+				dbgprintf("[COMPLEX]");
 			dbgprintf("\n");
 			pTpe = pTpe->pNext;
 		}
@@ -2130,8 +2187,6 @@ int tplGetEntryCount(struct template *pTpl)
 	return(pTpl->tpenElements);
 }
 
-/* our init function. TODO: remove once converted to a class
- */
 rsRetVal templateInit()
 {
 	DEFiRet;
